@@ -6,46 +6,44 @@
 
 ADF   <- function(x,                 # numeric vector related to the observed time series (or the series of residuals from a model)
                   dtype=c("ADF","CADF", "RPADF", "DeltaADF", "ACF"),         # options: ADF, CADF, RPADF, DeltaADF       
-                  lag.max,            # the maximum lag considered
+                  lag.max=floor(10 * log10(length(x))),            # the maximum lag considered
                   alpha=0.05,         # the significance level of the tests of lag independence (related to each bar)
                   num.clas,           # ADF:number of classes in each contingency table (as default, see equation (2) in the paper)
-                  alphaRP=0.5,        # RP:the coverage probability of the confidence interval for RP 
                   B=99,               # DeltaADF:number of permutation
                   bandwidth,          # DeltaADF:bandwidth
-                  delta="Delta_1",
-                                      # DeltaADF:option to select the divergence measure when dtype="Delta_ADF" is used
-                  fres=".Perm",      # DeltaADF:name of a function used to make permutations
+                  delta="Delta_1",    # DeltaADF:option to select the divergence measure when dtype="Delta_ADF" is used
+                  fres=".Perm",       # DeltaADF:name of a function used to make permutations
                   fdenest=".denest",  # DeltaADF:name of a function used for density estimation 
-                  fdiv,              # DeltaADF:name of a function used to compute divergence
-                  argacf,                # List with pptional acf arguments
+                  fdiv,          # DeltaADF:name of a function used to compute divergence
+                  argacf,        # List with pptional acf arguments
+                  R=1:lag.max,
+                  p.adjust.method =  p.adjust.methods,
                   plot=TRUE,          # Display plot
                  ...                  # plot method arguments
 ){
-  #require("Hmisc") 
-  #require("sm")
-  #require("rpanel")
-  #require("rgl")
   if(missing(x)) {stop("error: no value specified for argument x")} 
   if(missing(lag.max)) 
     lag.max <- floor(10 * log10(length(x)))  # rule of thumb
   lag.max <- min(lag.max, length(x) - 1)
-  
   if(lag.max < 0)  stop("'lag.max' must be at least 0")
-  
   dtype <- match.arg(dtype)
   delta <- match.arg(delta,c("Delta_1", "Delta_0.5", "Delta_2", "Delta_3", "Delta_4", "Delta_SD", "Delta_L1", "Delta_ST","Delta_fdiv"), several.ok=TRUE)
   series <- deparse(substitute(x))
-  result <- eval(parse(text=paste0("result <- .",dtype,".compute(x=x,lag.max=lag.max,num.clas=num.clas,alpha=alpha,B=B,dtype=dtype,alphaRP=alphaRP,bandwidth=bandwidth,fres=fres,fdenest=fdenest,delta=delta,fdiv=fdiv, argacf=argacf, plot=plot,series=series,...)")))
-                                               
-  if (!class(result)=="acf") {
-    class(result) <-"SDD"
-    if (plot) plot(result,...)
+  result <- eval(parse(text=paste0("result <- .",dtype,".compute(x=x,lag.max=lag.max,num.clas=num.clas,alpha=alpha,B=B,dtype=dtype,bandwidth=bandwidth,fres=fres,fdenest=fdenest,delta=delta,fdiv=fdiv, argacf=argacf, plot=plot,series=series, R= R,...)")))
+  class(result) <-"SDD"  
+  if (plot) {
+    if (is.null(list(...)$main)) plot(result, main="",...)
+    else plot(result,...)
   }
+  p.adjust.method <- match.arg(p.adjust.method)
+  result[["R"]] <- R
+  result[["p.adjust"]] <- p.adjust(result$res$pvalue[R], method = p.adjust.method)
+  result[["p.adjust.method"]] <-  p.adjust.method
   invisible(result)
 } 
 
 
-.ACF.compute<- function (x,lag.max,num.clas,alpha,B,dtype,bandwidth,argacf,main,plot,...){
+.ACF.compute<- function (x,lag.max,num.clas,alpha,B,dtype,bandwidth,argacf,main,plot,R,series,...){
   if (missing(argacf)) {argacf <- list()}
   argacf$x <- x
   argacf$plot <- FALSE
@@ -53,7 +51,6 @@ ADF   <- function(x,                 # numeric vector related to the observed ti
   
   acf$acf = acf$acf[-1, , , drop = FALSE]
   acf$lag = acf$lag[-1, , , drop = FALSE]
-  plot(acf,main = ifelse (missing(main), "", main))
   alpha <- ifelse (is.null(argacf$ci),0.05, 1 - argacf$ci)
   pvalue  <- 2 * (1 - pnorm( abs(acf$acf), 0, 1/sqrt( length(x) ) ) )
   pstar <- (pvalue<alpha)*(-1/(2*alpha)*pvalue+1)+(pvalue>=alpha)*(1-pvalue)/(2*(1-alpha))
@@ -66,7 +63,8 @@ ADF   <- function(x,                 # numeric vector related to the observed ti
     crit.val = qnorm(1-alpha/2)/sqrt(length(x)),
     n = (length(x)-1):(length(x)-max(acf$lag))
   )
-  acf$series <- deparse(substitute(x))
+  acf$Portmanteau.test <- 1-pchisq(sum(res$vbar[R]^2)*length(x),df=length(R))
+  acf$series <- series
   acf$res <- res
   acf$dtype    <- dtype
   acf$alpha   <- alpha
@@ -77,7 +75,7 @@ ADF   <- function(x,                 # numeric vector related to the observed ti
 ################################################################################
 #####  dtype ADF                                                        #########
 ################################################################################
-.ADF.compute <- function (x,lag.max,num.clas,alpha,B,dtype,series,...){
+.ADF.compute <- function (x,lag.max,num.clas,alpha,B,dtype,series,R,...){
   index <- length(x)
   if(missing(num.clas)){
     ks       <- floor(sqrt((index-lag.max)/5))
@@ -87,8 +85,6 @@ ADF   <- function(x,                 # numeric vector related to the observed ti
   
   if(num.clas>sqrt((index-lag.max)/5)){
     cat("warning:num.clas and/or lag.max maybe too large for the lenght of x\n")
-  #  num.clas <- floor(sqrt((index-lag.max)/5))
-  #  cat(paste("num.clas =",num.clas,"\n",sep=""))
   }
   
   tab.freq      <- NULL
@@ -127,12 +123,14 @@ ADF   <- function(x,                 # numeric vector related to the observed ti
     crit.val = critvalue,
     n=n
     )
+  Portmanteau.test <- 1-pchisq(q=sum(res$vbar[R]),df=(num.clas-1)^2*length(R))
   return(list(
       res      = res,
       dtype    = dtype,
       num.clas = num.clas,
       alpha    = alpha,
       df       = df,
+      Portmanteau.test = Portmanteau.test,
       series=series
     )
     )
@@ -141,13 +139,14 @@ ADF   <- function(x,                 # numeric vector related to the observed ti
 ################################################################################
 #####  dtype CADF                                                     #########
 ################################################################################
-.CADF.compute <- function (x,lag.max,num.clas,alpha,B,dtype,series,...){
-  ADF             <- .ADF.compute(x,lag.max,num.clas,alpha,B,dtype="ADF",series)
+.CADF.compute <- function (x,lag.max,num.clas,alpha,B,dtype,series,R,...){
+  ADF             <- .ADF.compute(x,lag.max,num.clas,alpha,B,dtype="ADF",series,R=R)
   CADF.ADF      <- sqrt(ADF$res$vbar/(ADF$res$n*(ADF$num.clas-1)))
   CADF.crit.val <- sqrt(ADF$res$crit.val/(ADF$res$n*(ADF$num.clas-1)))
   res <- data.frame(
     lag  = 1:lag.max,
     vbar    = CADF.ADF,
+    pvalue   = ADF$res$pvalue,
     crit.val=CADF.crit.val
   )
   return(list(
@@ -156,41 +155,31 @@ ADF   <- function(x,                 # numeric vector related to the observed ti
     num.clas = ADF$num.clas,
     alpha    = alpha,
     df       = ADF$df,
+    Portmanteau.test = ADF$Portmanteau.test,
     series=series
   ) )
-  return(ADF)  
 }
 ################################################################################
 #####  dtype RPADF                                                      #########
 ################################################################################
-.RPADF.compute <- function (x,lag.max,num.clas,alpha,B,dtype,alphaRP,fres,series,...){  
+.RPADF.compute <- function (x,lag.max,num.clas,alpha,B,dtype,fres,series,...){ 
 
-    ADF     <- .ADF.compute(x,lag.max,num.clas,alpha,B,dtype="ADF",series)
+    ADF     <- .ADF.compute(x,lag.max,num.clas,alpha,B,dtype="ADF",series,R=1)
     
     lambda     <- numeric(lag.max)
-    Lower      <- numeric(lag.max)
-    Upper      <- numeric(lag.max)
     RP         <- numeric(lag.max)
-    RPL        <- numeric(lag.max)
-    RPU        <- numeric(lag.max)
     
     lambda     <-  sapply(1:lag.max,.fmin,ADF,k=1.5,g=0.5)
-    Lower      <-  sapply(1:lag.max,.fmin,ADF,k=2,g=1-alphaRP/2)
-    Upper      <-  sapply(1:lag.max,.fmin,ADF,k=2,g=alphaRP/2)
  
     RP         <- 1-pchisq(qchisq(1-alpha,df=ADF$df),df=ADF$df,ncp=lambda)
-    RPL        <- 1-pchisq(qchisq(1-alpha,df=ADF$df),df=ADF$df,ncp=Lower)
-    RPU        <- 1-pchisq(qchisq(1-alpha,df=ADF$df),df=ADF$df,ncp=Upper)
     
     critvalue  <- rep(qchisq(1-alpha,df=ADF$df[1]),lag.max)
        
     res <- data.frame(
       lag  = 1:lag.max,
       vbar    = RP,
-      lower   = RPL,
-      upper = RPU,
-      xmin = lambda,
-      alphaRP = rep(alphaRP, lag.max)
+      pvalue   = ADF$res$pvalue,      
+      xmin = lambda
     )
     return(list(
       res      = res,
@@ -198,6 +187,7 @@ ADF   <- function(x,                 # numeric vector related to the observed ti
       num.clas = ADF$num.clas,
       alpha    = alpha,
       df       = ADF$df,
+      Portmanteau.test = ADF$Portmanteau.test,
       series=series
     ) )
 }  
@@ -211,32 +201,31 @@ ADF   <- function(x,                 # numeric vector related to the observed ti
 ################################################################################
 #####  dtype DeltaADF                                                   #########
 ################################################################################
-.DeltaADF.compute <- function (x,lag.max,num.clas,alpha,B,dtype,alphaRP,bandwidth, fres, fdenest,delta,fdiv,series,...)
+.DeltaADF.compute <- function (x,lag.max,num.clas,alpha,B,dtype,bandwidth, fres, fdenest,delta,fdiv,series,R=1:lag.max,...)
 { 
-  P           <- eval(parse(text=paste(fres,"(x,B=B)"))) #Perm(x,B=B) # (B x n)-matrix of permuted samples
- 
-  # bandwidth determination (LCV method)
   if (missing(bandwidth))  bandwidth <- .lcv(x)
-  
-  pvalue      <- numeric(lag.max)
-  result <- list()
-  for (i in 1:length(delta)){
-    pvalue <-  sapply(1:lag.max,.p_value,x,P=P,delta=delta[i],fdenest=fdenest,bandwidth=bandwidth,fdiv=fdiv)
-    pstar  <- (pvalue<alpha)*(-1/(2*alpha)*pvalue+1)+(pvalue>=alpha)*(1-pvalue)/(2*(1-alpha))
-    res    <- data.frame(
+  P         <- eval(parse(text=paste(fres,"(x,B=B)"))) #Perm(x,B=B) # (B x n)-matrix of permuted samples
+  pvalue    <- numeric(lag.max)
+  result    <- list()
+      deltap <- sapply(1:lag.max,.p_value,x,P=P,delta=delta,fdenest=fdenest,bandwidth=bandwidth,fdiv=fdiv)
+      deltapm   <- matrix(unlist(deltap),nrow=(B+2),ncol=lag.max) 
+      pvalue <- deltapm[1,]
+      pstar  <- (pvalue<alpha)*(-1/(2*alpha)*pvalue+1)+(pvalue>=alpha)*(1-pvalue)/(2*(1-alpha))
+      Portmanteau.test <- ((rank(-rowSums(deltapm[2:(B+2),R]),ties.method="random")-1)/B)[1] #Portmanteau-test
+      res    <- data.frame(
       lag  = 1:lag.max,
       vbar = pstar,
       pvalue   = pvalue
       )
-    result[[i]] <- list(
+    result <- list(
       res      = res,
       dtype     = dtype,
-      delta     = delta[i],
+      delta     = delta,
       alpha    = alpha,
       bandwidth = bandwidth,
+      Portmanteau.test = Portmanteau.test,
       series=series
     )
-  }
   return(result)
   
 }
@@ -295,11 +284,11 @@ ADF   <- function(x,                 # numeric vector related to the observed ti
   B           <- nrow(P)    # number of permutations
   Svector     <- numeric(B)
   Svector <- sapply(1:B,.stattestKERN, x=P,lag=lag,bandwidth=bandwidth,delta=delta,fdenest=fdenest, fdiv=fdiv)
-  Svector <- (rank(-Svector,ties.method="random")-1)/B
-  p.value <- Svector[1]
-  
-  return(p.value=p.value)
+  Svector1 <- (rank(-Svector,ties.method="random")-1)/B
+  p.value <- Svector1[1]
   
+  return(list(p.value=p.value,Svector=Svector))
+  #return(p.value=p.value)
 }
 
 
@@ -393,11 +382,7 @@ plot.SDD <- function(x,
                      step=5,             # the step between x-ticks (default=5) 
                      ...)
 {
-  if (is.null(x[[1]]$dtype)) x <- list(x)
-  for (i in 1:length(x)){
-    eval(parse(text=paste0(".",x[[i]]$dtype,".plot(x[[i]],norm=norm,stability=stability,step=step,...)")))
-    if (i<length(x)) dev.new()
-  }
+    eval(parse(text=paste0(".",x$dtype,".plot(x,norm=norm,stability=stability,step=step,...)")))
 }  
 
 .ADF.plot <- function(x,norm,stability,step,ylim,ylab,...){
@@ -414,6 +399,14 @@ plot.SDD <- function(x,
     abline(a=x$res$crit.val,b=0,col="blue",lty=2)
   }
 }
+.ACF.plot <- function(x,norm,stability,step,ylim,ylab,...){
+    if (missing(ylab)) ylab <- "ACF"
+    if(missing(ylim)) ylim <- c(min(-1.2*x$res$crit.val,1.2*x$res[[2]]),max(1.2*x$res$crit.val,1.2*x$res[[2]]))
+    .gen.plot(x, step=step, ylim=ylim, ylab=ylab,...)
+    abline(a=x$res$crit.val,b=0,col="blue",lty=2)
+    abline(a=-x$res$crit.val,b=0,col="blue",lty=2)
+}
+
 .CADF.plot <-  function(x,norm,stability,step,ylim,ylab,...){
   if (missing(ylab)) ylab <- "CADF"
   .gen.plot(x,ylab=ylab,step=step, ylim=ylim,...)
@@ -427,9 +420,6 @@ plot.SDD <- function(x,
   if (stability){
     lag.max <- max(x$res$lag)
     rect(1:lag.max-0.21, rep(0,lag.max), 1:lag.max+0.21, x$res$vbar, density = NULL, angle = 45, border = NULL, lty=1, lwd=2, col="gray")
-    segments(1:lag.max, x$res$lower, 1:lag.max, x$res$upper)
-    segments(1:lag.max-0.1, x$res$lower, 1:lag.max+0.1, x$res$lower)
-    segments(1:lag.max-0.1, x$res$upper, 1:lag.max+0.1, x$res$upper)
   }
 }
 .DeltaADF.plot <-  function(x,norm,stability,step,ylim,ylab,...){
@@ -457,17 +447,19 @@ plot.SDD <- function(x,
 
 print.SDD <- function(x, digits=3, ...)
 {
-   invisible(x)
-  if (is.null(x[[1]]$dtype)) x <- list(x)
-  type <- match(x[[1]]$dtype, c("ADF","CADF", "RPADF", "DeltaADF","acf"))
-  msg <- c("Autodependogram", "CADF Contingency Coefficients Autodependogram", "Reproducibility Probability Autodependogram", "Divergence-based Autopedendograms", "Autocorrelation")
-  cat("\n", msg[type], " of series ", sQuote(x[[1]]$series)," by lag\n", sep="")
-  for (i in 1:length(x)) {
-    if (!is.null(x[[i]]$delta)) cat("\n",paste0("Delta[",strsplit(x[[i]]$delta,"_")[[1]][2],"]-ADF\n", sep=""))
-    adf <-drop (x[[i]]$res[[2]])
-    names(adf) <- format(x[[i]]$res[[1]])
-    print(adf,digits = digits, ...)
-  }
+  invisible(x)
+  msg <- paste0(x$dtype, " bars ",ifelse(!is.null(x$delta),paste0("(with method ",x$delta,") "),""),"for series '",x$series,"'")
+  cat("\n",msg,"\n")
+  adf <-drop (x$res[[2]])
+  names(adf) <- format(x$res[[1]])
+  print(adf,digits = digits, ...)
+  cat("\nSimultaneous Test \n")
+  cat("  adjustment method:", x$p.adjust.method)
+  cat("\n  adjusted p-value: ", round(min(x$p.adjust),3),"\n",sep="")
+  cat("\nPortmanteau Test ")
+  if (x$dtype %in% c("ACF")) cat("(Box-Pierce test)")
+  cat("\n  p-value: ", round(x$Portmanteau.test,3),"\n",sep="")
+  cat(paste0("\nTested lags \n  ",paste(x$R,collapse = ","), "\n"))
 }
 
 
